# -*- perl -*-
#
# Copyright (C) 1996 Ken MacLeod
# See the file COPYING for distribution terms.
#

use strict;

package Text::GDO::Transform;

sub map_instance {
    my ($self, $instance) = @_;
    my ($mapped_instance) = {};

    # This is defined here as a dummy function to replace SGMLS'
    # original definition for this routine 
    # XXX need only be done once.  does SGMLSpm break without this if
    # I don't specifically use this method?
    eval q{package SGMLS_Element; sub parse { return bless {}, 'Text::GDO::SGML::Element'; } 1;}
        || die "$::prog: oops, $@\n";
    push (@{$mapped_instance->{diversion}}, {});
    $self->_map_instance ($mapped_instance, $instance->{content});
    $mapped_instance->{content} = pop (@{$mapped_instance->{diversion}});

    return ($mapped_instance->{content}{content}[0]);
}

sub _map_instance {
    my ($transform, $mapped_instance, $element) = @_;
    my ($event);
    my ($my_type) = ref $transform;
    my ($event_type);

    foreach $event (@{$element->[4]}) {
	$event_type = $event->type;
	if ($event_type eq 'start_element') {
	    my ($element_name) = $event->data->name;
	    $element_name =~ tr/a-z/A-Z/;
	    my ($map_element) = "${my_type}::map"
		. $transform->{'map'}{$element_name};
	    no strict 'refs';	# use of symbolic ref is intentional
	    if (defined $transform->{'map'}{$element_name}
		&& defined &$map_element) {
		&$map_element ($transform, $mapped_instance, $event, $element);
	    } else {
		if (!$transform->{warned}{$element_name}) {
		    warn "$::prog: No mapping for element \`$element_name'\n";
		    $transform->{warned}{$element_name} = 1;
		}
	    }
	    use strict 'refs';
	} elsif ($event_type eq 'cdata'  || $event_type eq 'sdata'
		 || $event_type eq 're' || $event_type eq 'pi'
		 || $event_type eq 'entity') {
	    push (@{$mapped_instance->{'diversion'}[-1]{'content'}},
		  new Text::GDO::Data ('parent' => $mapped_instance->{'diversion'}[-1],
			    'source' => $event));
	}
    }
}

sub map_gi {
    my ($self, %new_mapping) = @_;
    my ($key, $value, @keys);

    while (($key, $value) = each %new_mapping) {
	unshift (@{$self->{'map_stack'}{$key}}, $self->{'map'}{$key});
	$self->{'map'}{$key} = $value;
    }
    @keys = keys %new_mapping;
    push (@{$self->{'mappings_stack'}}, \@keys);
}

sub unmap_gi {
    my ($self) = @_;
    my ($key, $keys, @keys);

    $keys = pop (@{$self->{'mappings_stack'}});
    @keys = @{$keys};
    foreach $key (@keys) {
	$self->{'map'}{$key} = shift (@{$self->{'map_stack'}{$key}});
    }
}

1;
